package com.magic.creator;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.nio.charset.Charset;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.cyberneko.html.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.magic.creator.abilities.Ability;
import com.magic.creator.abilities.SpellAbility;
import com.magic.creator.abilities.StaticAbility;
import com.magic.creator.abilities.TriggeredAbility;
import com.magic.creator.cards.Card;
import com.magic.creator.enums.CardTypeEnum;
import com.magic.creator.enums.ExpansionEnum;
import com.magic.creator.enums.LanguageEnum;
import com.magic.creator.enums.RarityEnum;
import com.magic.creator.misc.CastingCost;
import com.sun.org.apache.xpath.internal.XPathAPI;

public class Creator {
	public static void main(String[] args) throws UnknownHostException, IOException {

		String filename = args.length > 0 ? args[0] : "input.txt";
		FileInputStream input = new FileInputStream(filename);
		List<String> readLines = IOUtils.readLines(input);
		input.close();

		for (String tmpCardName : readLines) {

			String[] cardNames = tmpCardName.split("\\|");

			for (String cardName : cardNames) {
				String cardUrl = URLEncoder.encode(cardName, "UTF8");
				String url = "http://magiccards.info/query?q=%2B%2Bo%21%22" + cardUrl + "%22&v=spoiler&s=cname";
				Card card = xget(url, "//TD");
				if (card != null) {
					System.out.println("OK:" + cardName);
					card.setExpansion(ExpansionEnum.DPG);
					FileOutputStream output = new FileOutputStream("out/" + card.getCardName() + "_XXXXX" + ".xml");
					output.write(card.toString().getBytes(Charset.forName("UTF8")));
					output.flush();
					output.close();
				} else {
					System.err.println("KO COULDN'T FIND:" + cardName);
				}
			}
		}
		System.out.println("\nFinished !\n");
	}

	public static Card xget(String url, String exp) {

		Map<ExpansionEnum, Card> cards = new HashMap<ExpansionEnum, Card>();

		try {
			DOMParser parser = new DOMParser();
			parser.parse(url);
			Document doc = parser.getDocument();
			NodeList selectNodeList = XPathAPI.selectNodeList(doc, exp);

			for (int i = 0; i < selectNodeList.getLength(); i++) {
				Node item = selectNodeList.item(i);

				Node linkNode = XPathAPI.selectSingleNode(item, "SPAN/A");
				if (linkNode != null) {

					String info[] = linkNode.getAttributes().getNamedItem("href").getNodeValue().split("/");
					ExpansionEnum edition = ExpansionEnum.get(info[1]);
					LanguageEnum language = LanguageEnum.get(info[2]);

					// Card initialization
					Card card = cards.get(edition);
					if (card == null) {
						card = new Card();
						cards.put(edition, card);
						String collectorNumberString = info[3].split("\\.")[0].replace("a", "").replace("b", "")
								.replace("c", "").replace("d", "").replace("e", "");
						try {
							card.setCollectorNumber(Integer.parseInt(collectorNumberString));
						} catch (Exception e) {
							System.err.println("Error parsing collector number: " + collectorNumberString);
						}
						card.setRarity(RarityEnum.get(XPathAPI.selectSingleNode(item, "P/I").getTextContent()));
						card.setExpansion(edition);
					}

					// Set localized title + card_name
					String title = linkNode.getTextContent();
					card.getTitle().add(language, title);
					if (language.equals(LanguageEnum.EN)) {
						String cardName = title.toUpperCase();
						cardName = cardName.replace(' ', '_');
						cardName = cardName.replace(",", "");
						cardName = cardName.replace("'", "");
						card.setCardName(cardName);
					}

					NodeList pNodes = XPathAPI.selectNodeList(item, "P");

					if (language.equals(LanguageEnum.EN)) {
						Node typeNode = pNodes.item(1);
						String[] typeInfo = typeNode.getTextContent().split("\\n");
						String[] types = typeInfo[0].trim().split(" ");
						card.setCastingCost(new CastingCost(typeInfo[1].trim().split(" ")[0]));

						for (String type : types) {
							type = type.replace(",", "").trim();
							CardTypeEnum cardType = CardTypeEnum.get(type);
							if (!cardType.equals(CardTypeEnum.UNKNOWN)) {
								card.getCardTypes().add(cardType);
							} else if (type.equals("Legendary")) {
								card.setLegendary(true);
							} else {
								if (type.contains("/")) {
									String[] powTou = type.split("/");
									card.setPower(powTou[0]);
									card.setToughness(powTou[1]);
								} else if (!type.equals("")) {
									card.getTypes().add(type);
								}
							}
						}
					}

					// Artist
					Node artistNode = pNodes.item(pNodes.getLength() - 1);
					card.setArtist(artistNode.getTextContent().replace("Illus. ", "").trim());

					// Flavour text
					Node flavourNode = pNodes.item(pNodes.getLength() - 2);
					String flavour = flavourNode.getTextContent().trim();
					if (!flavour.isEmpty()) {
						card.getFlavourText().add(language, flavour);
					}

					// Find abilities
					Node selectSingleNode = XPathAPI.selectSingleNode(item, "P/B");
					int abilityIndex = 0;
					NodeList abilityNodes = selectSingleNode.getChildNodes();
					for (int j = 0; j < abilityNodes.getLength(); j++) {
						Node abilityNode = abilityNodes.item(j);
						if (abilityNode.getNodeName().equals("#text")) {
							Ability ability;
							String abilityText = abilityNode.getNodeValue();
							if (abilityIndex >= card.getAbilities().size()) {
								if (card.getCardTypes().contains(CardTypeEnum.INSTANT)
										|| card.getCardTypes().contains(CardTypeEnum.SORCERY)) {
									ability = new SpellAbility();
								} else {
									if (abilityText.contains("}:") || abilityText.contains("} :")) {
										ability = new TriggeredAbility();
									} else {
										ability = new StaticAbility();
									}
								}
								card.getAbilities().add(ability);
							} else {
								ability = card.getAbilities().get(abilityIndex);
							}

							ability.getText().add(language, abilityText);
							abilityIndex++;
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		// Find the "best" card : most translations, and flavor text if
		// available
		if (cards.size() == 0) {
			return null;
		}

		Collection<Card> values = cards.values();
		Card bestCard = values.iterator().next();
		for (Card card : values) {
			if (card.getTitle().getLabelCount() > bestCard.getTitle().getLabelCount()
					|| (card.getTitle().getLabelCount() == bestCard.getTitle().getLabelCount() && card.getFlavourText()
							.getLabelCount() > bestCard.getFlavourText().getLabelCount()))
				bestCard = card;
		}
		return bestCard;
	}
}
