package com.magic.creator.cards;

import java.util.ArrayList;
import java.util.List;

import com.magic.creator.abilities.Ability;
import com.magic.creator.enums.CardTypeEnum;
import com.magic.creator.enums.ExpansionEnum;
import com.magic.creator.enums.RarityEnum;
import com.magic.creator.misc.CastingCost;
import com.magic.creator.misc.Help;
import com.magic.creator.misc.Text;

public class Card {

	private final List<String> types = new ArrayList<String>();
	private String cardName = "";
	private final Text title = new Text();
	private String mutiverseId = "?????";
	private String artId = "?????";
	private String artist = "";
	private CastingCost castingCost = new CastingCost("0");
	private final Text flavourText = new Text();
	private final List<CardTypeEnum> cardTypes = new ArrayList<CardTypeEnum>();
	private ExpansionEnum expansion = ExpansionEnum.UNKNOWN;
	private int collectorNumber = 0;
	private RarityEnum rarity = RarityEnum.UNKNOWN;
	private String power = "";
	private String toughness = "";
	private final List<Ability> abilities = new ArrayList<Ability>();
	private final List<Help> helps = new ArrayList<Help>();
	private boolean legendary;

	public List<String> getTypes() {
		return types;
	}

	public String getCardName() {
		return cardName;
	}

	public void setCardName(String cardName) {
		this.cardName = cardName;
	}

	public Text getTitle() {
		return title;
	}

	public String getMutiverseId() {
		return mutiverseId;
	}

	public void setMutiverseId(String mutiverseId) {
		this.mutiverseId = mutiverseId;
	}

	public String getArtId() {
		return artId;
	}

	public void setArtId(String artId) {
		this.artId = artId;
	}

	public String getArtist() {
		return artist;
	}

	public void setArtist(String artist) {
		this.artist = artist;
	}

	public CastingCost getCastingCost() {
		return castingCost;
	}

	public void setCastingCost(CastingCost castingCost) {
		this.castingCost = castingCost;
	}

	public Text getFlavourText() {
		return flavourText;
	}

	public List<CardTypeEnum> getCardTypes() {
		return cardTypes;
	}

	public ExpansionEnum getExpansion() {
		return expansion;
	}

	public void setExpansion(ExpansionEnum expansion) {
		this.expansion = expansion;
	}

	public RarityEnum getRarity() {
		return rarity;
	}

	public void setRarity(RarityEnum rarity) {
		this.rarity = rarity;
	}

	public String getPower() {
		return power;
	}

	public void setPower(String power) {
		this.power = power;
	}

	public String getToughness() {
		return toughness;
	}

	public void setToughness(String toughness) {
		this.toughness = toughness;
	}

	public List<Ability> getAbilities() {
		return abilities;
	}

	public int getCollectorNumber() {
		return collectorNumber;
	}

	public void setCollectorNumber(int collectorNumber) {
		this.collectorNumber = collectorNumber;
	}

	public List<Help> getHelps() {
		return helps;
	}

	public boolean isLegendary() {
		return legendary;
	}

	public void setLegendary(boolean legendary) {
		this.legendary = legendary;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append("<?xml version='1.0'?>\n");
		stringBuilder.append("<CARD_V2>\n");
		stringBuilder.append("  <FILENAME text=\"" + getCardName() + "_" + getMutiverseId() + "\" />\n");
		stringBuilder.append("  <CARDNAME text=\"" + getCardName() + "\" />\n");
		stringBuilder.append("  <TITLE>\n");
		stringBuilder.append(getTitle());
		stringBuilder.append("  </TITLE>\n");
		stringBuilder.append("  <MULTIVERSEID value=\"" + getMutiverseId() + "\" />\n");
		stringBuilder.append("  <ARTID value=\"" + getArtId() + "\" />\n");
		stringBuilder.append("  <ARTIST name=\"" + getArtist() + "\" />\n");

		stringBuilder.append("  <CASTING_COST cost=\""
				+ (getCardTypes().contains(CardTypeEnum.LAND) ? "" : getCastingCost()) + "\" />\n");

		if (getFlavourText().hasText()) {
			stringBuilder.append("  <FLAVOURTEXT>\n");
			stringBuilder.append(getFlavourText());
			stringBuilder.append("  </FLAVOURTEXT>\n");
		}

		if (isLegendary()) {
			stringBuilder
					.append("  <SUPERTYPE metaname=\"Legendary\" order_de-DE=\"0\" order_es-ES=\"1\" order_fr-FR=\"1\" order_it-IT=\"1\" order_jp-JA=\"0\" order_ko-KR=\"0\" order_pt-BR=\"0\" order_ru-RU=\"0\" />\n");
		}

		for (CardTypeEnum cardType : getCardTypes()) {
			stringBuilder.append("  <TYPE metaname=\"" + cardType + "\" />\n");
		}

		int subTypeIndex = 0;
		for (String type : getTypes()) {
			stringBuilder.append("  <SUB_TYPE metaname=\"" + type + "\" order_de-DE=\"" + subTypeIndex
					+ "\" order_es-ES=\"" + subTypeIndex + "\" order_fr-FR=\"" + subTypeIndex + "\" order_it-IT=\""
					+ subTypeIndex + "\" order_jp-JA=\"" + subTypeIndex + "\" order_ko-KR=\"" + subTypeIndex
					+ "\" order_pt-BR=\"" + subTypeIndex + "\" order_ru-RU=\"" + subTypeIndex + "\" />\n");
			subTypeIndex++;
		}

		stringBuilder.append("  <EXPANSION value=\"" + getExpansion().name() + "\" />\n");
		stringBuilder.append("  <RARITY metaname=\"" + getRarity() + "\" />\n");

		if (getCardTypes().contains(CardTypeEnum.CREATURE)) {
			stringBuilder.append("  <POWER value=\"" + getPower() + "\" />\n");
			stringBuilder.append("  <TOUGHNESS value=\"" + getToughness() + "\" />\n");
		}

		for (Ability ability : getAbilities()) {
			stringBuilder.append(ability);
		}

		for (Help help : getHelps()) {
			stringBuilder.append(help);

		}

		stringBuilder.append("</CARD_V2>\n");

		return stringBuilder.toString();
	}
}
